package com.engine.salary.remote.cbs8.client;

import cn.hutool.core.util.StrUtil;
import com.engine.salary.exception.CBS8RunTimeException;
import com.engine.salary.remote.cbs8.common.Constants;
import com.engine.salary.remote.cbs8.response.GetTokenResponse;
import com.engine.salary.remote.cbs8.util.SM2Util;
import com.engine.salary.remote.cbs8.HttpUtil;
import com.engine.salary.remote.cbs8.JsonUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.io.IOUtils;
import org.apache.http.Header;
import org.apache.http.HttpMessage;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.ByteArrayEntity;
import org.apache.http.protocol.HTTP;
import weaver.general.BaseBean;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import java.util.zip.GZIPInputStream;

@Slf4j
public class CBS8BaseClient {

    static final String appId = new BaseBean().getPropValue("hrmSalaryCBS8", "app_id");

    static final String appSecret = new BaseBean().getPropValue("hrmSalaryCBS8", "app_secret");

    /**
     * 财资管理云公钥(平台公钥)
     */
    static final String bodyEncryptionKey = new BaseBean().getPropValue("hrmSalaryCBS8", "bodyEncryptionKey");

    /**
     * 企业私钥（加密）
     */
    static final String signEncryptionPrivateKey = new BaseBean().getPropValue("hrmSalaryCBS8", "signEncryptionPrivateKey");

    /**
     * 企业私钥（解密）
     */
    static final String bodyDecryptionKey = new BaseBean().getPropValue("hrmSalaryCBS8", "bodyDecryptionKey");

    /**
     * 域名
     */
    static final String host = new BaseBean().getPropValue("hrmSalaryCBS8", "host");


    /**
     * 获取token
     *
     * @return
     */
    public static String getToken() {
        Map<String, String> params = new HashMap<>();
        params.put("app_id", appId);
        params.put("app_secret", appSecret);
        params.put("grant_type", "client_credentials");
        //  开始请求
        String paramBody = JsonUtil.toJsonString(params);
        String res = HttpUtil.doPost(host + "/openapi/app/v1/app/token", new HashMap<>(), paramBody, HttpUtil.JSON_TYPE);
        GetTokenResponse getTokenResponse = JsonUtil.parseBean(res, GetTokenResponse.class);
        log.info("获取token params:{} res: {}", paramBody, res);
        if (Objects.isNull(getTokenResponse)) {
            throw new CBS8RunTimeException("服务异常");
        }
        if (!"0".equals(getTokenResponse.getCode())) {
            throw new CBS8RunTimeException(getTokenResponse.getMsg());
        }
        if (getTokenResponse.getData() == null) {
            throw new CBS8RunTimeException("未获取数据");
        }
        if (StrUtil.isBlank(getTokenResponse.getData().getToken())) {
            throw new CBS8RunTimeException("未获取token");
        }
        String token = getTokenResponse.getData().getToken();
        return token;
    }


    /**
     * 生成请求报文
     */
    public static HttpPost setupRequest(String url, String requestData) {

        long timestamp = System.currentTimeMillis();

        // 请求数据拼接：  报文体+时间戳
        byte[] requestDataBytes = requestData.getBytes(StandardCharsets.UTF_8);
        byte[] timestampBytes = ("&timestamp=" + timestamp).getBytes(StandardCharsets.UTF_8);
        byte[] newBytes = new byte[requestDataBytes.length + timestampBytes.length];
        System.arraycopy(requestDataBytes, 0, newBytes, 0, requestDataBytes.length);
        System.arraycopy(timestampBytes, 0, newBytes, requestDataBytes.length, timestampBytes.length);

        // 生成签名
        byte[] signature = SM2Util.sign(signEncryptionPrivateKey, newBytes);
        String sign = Base64.encodeBase64String(SM2Util.encodeDERSignature(signature));

        // 设置请求URL
        HttpPost httpPost = new HttpPost(url);
        // 请求头设置签名
        httpPost.setHeader(Constants.SIGN_HEADER_NAME, sign);
        // 请求头设置时间戳
        httpPost.setHeader(Constants.TIMESTAMP_HEADER, Long.toString(timestamp));
        // 请求头设置请求参数格式，请根据实际情况改写
        httpPost.setHeader(HTTP.CONTENT_TYPE, Constants.TARGET_CONTENT_TYPE);
        // 请求头设置TOKEN
        String token = getToken();
        httpPost.setHeader(Constants.AUTHORIZATION, Constants.BEARER + token);

        // 报文体加密
        byte[] encryptedData = SM2Util.encrypt(bodyEncryptionKey, requestDataBytes);
        // 设置请求体
        httpPost.setEntity(new ByteArrayEntity(encryptedData));

        return httpPost;
    }

    /**
     * 处理响应报文
     */
    public byte[] handleResponse(HttpResponse response) throws IOException {
        InputStream content = response.getEntity().getContent();
        byte[] responseData = IOUtils.toByteArray(content);

        if (responseData == null || responseData.length == 0) {
            return responseData == null ? new byte[0] : responseData;
        }

        // 步骤1 原始响应报文解密 如果服务网关获取加解密密钥失败，则无法解密请求报文，且无法加密响应报文。 这时候，网关会直接返回错误信息，响应报文是未加密状态。
        Boolean encryptionEnable = getHeader(response, Constants.ENCRYPTION_ENABLED_HEADER_NAME);

        if (Boolean.TRUE.equals(encryptionEnable)) {
            responseData = SM2Util.decrypt(bodyDecryptionKey, responseData);
        }

        Boolean xMbcloudCompress = getHeader(response, Constants.X_MBCLOUD_COMPRESS);
        if (Boolean.TRUE.equals(xMbcloudCompress)) {
            responseData = decompress(responseData);
        }

        return responseData;


    }

    private static Boolean getHeader(HttpMessage message, String name) {
        Header header = message.getFirstHeader(name);
        return header != null;
    }

    public static byte[] decompress(byte[] data) throws IOException {
        ByteArrayInputStream input = new ByteArrayInputStream(data);
        GZIPInputStream gzipInput = new GZIPInputStream(input);
        return IOUtils.toByteArray(gzipInput);
    }
}
